package de.hhhammer.dchat.bot.openai;

import de.hhhammer.dchat.bot.openai.MessageContext.PreviousInteraction;
import de.hhhammer.dchat.bot.openai.MessageContext.ReplyInteraction;
import de.hhhammer.dchat.bot.openai.models.ChatGPTRequest;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class ChatGPTRequestBuilder {
    private static final String smallContextModel = "gpt-3.5-turbo";
    private static final String bigContextModel = "gpt-3.5-turbo-16k";

    public ChatGPTRequestBuilder() {
    }

    public ChatGPTRequest contextRequest(List<? extends MessageContext> contextMessages, String message, String systemMessage) {
        List<ChatGPTRequest.Message> messages = getMessages(contextMessages, message, systemMessage);
        String contextModel = contextMessages.size() <= 1 ? smallContextModel : bigContextModel;
        return new ChatGPTRequest(
                contextModel,
                messages,
                0.7f
        );
    }

    @NotNull
    private static List<ChatGPTRequest.Message> getMessages(List<? extends MessageContext> contextMessages, String message, String systemMessage) {
        var systemMsg = new ChatGPTRequest.Message("system", systemMessage);
        List<ChatGPTRequest.Message> contextMsgs = getContextMessages(contextMessages);
        var userMessage = new ChatGPTRequest.Message("user", message);
        List<ChatGPTRequest.Message> messages = new ArrayList<>();
        messages.add(systemMsg);
        messages.addAll(contextMsgs);
        messages.add(userMessage);
        return messages;
    }

    @NotNull
    private static List<ChatGPTRequest.Message> getContextMessages(List<? extends MessageContext> contextMessages) {
        return contextMessages.stream()
                .map(ChatGPTRequestBuilder::mapContextMessages)
                .flatMap(List::stream)
                .toList();
    }

    @NotNull
    private static List<ChatGPTRequest.Message> mapContextMessages(MessageContext contextMessage) {
        return switch (contextMessage) {
            case PreviousInteraction previousInteractions -> List.of(
                    new ChatGPTRequest.Message("user", previousInteractions.question()),
                    new ChatGPTRequest.Message("assistant", previousInteractions.answer())
            );
            case ReplyInteraction replyInteractions ->
                    List.of(new ChatGPTRequest.Message("assistant", replyInteractions.answer()));
        };
    }
}
