package de.hhhammer.dchat.bot.openai;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.hhhammer.dchat.bot.openai.models.ChatGPTRequest;
import de.hhhammer.dchat.bot.openai.models.ChatGPTResponse;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.Duration;

public class ChatGPTService {
    private static final String url = "https://api.openai.com/v1/chat/completions";
    private final String apiKey;
    private final HttpClient httpClient;
    private final ObjectMapper mapper;

    public ChatGPTService(String apiKey, HttpClient httpClient, ObjectMapper mapper) {
        this.apiKey = apiKey;
        this.httpClient = httpClient;
        this.mapper = mapper;
    }

    public ChatGPTResponse submit(ChatGPTRequest chatGPTRequest) throws IOException, InterruptedException, ResponseException {
        var data = mapper.writeValueAsBytes(chatGPTRequest);
        var request = HttpRequest.newBuilder(URI.create(url))
                .POST(HttpRequest.BodyPublishers.ofByteArray(data))
                .setHeader("Content-Type", "application/json")
                .setHeader("Authorization", "Bearer " + this.apiKey)
                .timeout(Duration.ofMinutes(5))
                .build();

        var responseStream = httpClient.send(request, HttpResponse.BodyHandlers.ofInputStream());
        if (responseStream.statusCode() != 200) {
            throw new ResponseException("Response status code was not 200: " + responseStream.statusCode());
        }

        return mapper.readValue(responseStream.body(), ChatGPTResponse.class);
    }
}
