package de.hhhammer.dchat.web.server;

import de.hhhammer.dchat.db.DBException;
import de.hhhammer.dchat.db.ServerDBService;
import de.hhhammer.dchat.db.models.server.ServerConfig;
import io.javalin.apibuilder.CrudHandler;
import io.javalin.http.Context;
import io.javalin.http.HttpStatus;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConfigCrudHandler implements CrudHandler {
    private static final Logger logger = LoggerFactory.getLogger(ConfigCrudHandler.class);

    private final ServerDBService serverDBService;

    public ConfigCrudHandler(ServerDBService serverDBService) {
        this.serverDBService = serverDBService;
    }

    @Override
    public void create(@NotNull Context context) {
        var body = context.bodyAsClass(ServerConfig.NewServerConfig.class);
        try {
            this.serverDBService.addConfig(body);
        } catch (DBException e) {
            logger.error("Adding new server configuration", e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
        context.status(HttpStatus.CREATED);
    }

    @Override
    public void delete(@NotNull Context context, @NotNull String s) {
        try {
            this.serverDBService.deleteConfig(Long.parseLong(s));
            context.status(HttpStatus.NO_CONTENT);
        } catch (DBException e) {
            logger.error("Deleting configuration with id: " + s, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void getAll(@NotNull Context context) {
        try {
            var allowedServers = this.serverDBService.getAllConfigs();
            context.json(allowedServers);
        } catch (DBException e) {
            logger.error("Getting all server configs", e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void getOne(@NotNull Context context, @NotNull String s) {
        var id = Long.parseLong(s);
        try {
            var server = this.serverDBService.getConfigBy(id);
            if (server.isEmpty()) {
                context.status(HttpStatus.NOT_FOUND);
                return;
            }
            context.json(server.get());
        } catch (DBException e) {
            logger.error("Searching for config with id: " + s, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void update(@NotNull Context context, @NotNull String idString) {
        var body = context.bodyAsClass(ServerConfig.NewServerConfig.class);
        var id = Long.parseLong(idString);

        try {
            this.serverDBService.updateConfig(id, body);
        } catch (DBException e) {
            logger.error("Updating allowed server with id: " + idString, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
