package de.hhhammer.dchat.web.user;

import de.hhhammer.dchat.db.DBException;
import de.hhhammer.dchat.db.UserDBService;
import de.hhhammer.dchat.db.models.user.UserConfig;
import io.javalin.apibuilder.CrudHandler;
import io.javalin.http.Context;
import io.javalin.http.HttpStatus;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConfigUserCrudHandler implements CrudHandler {
    private static final Logger logger = LoggerFactory.getLogger(ConfigUserCrudHandler.class);

    private final UserDBService userDBService;

    public ConfigUserCrudHandler(UserDBService userDBService) {
        this.userDBService = userDBService;
    }

    @Override
    public void create(@NotNull Context context) {
        var body = context.bodyAsClass(UserConfig.NewUserConfig.class);
        try {
            this.userDBService.addConfig(body);
        } catch (DBException e) {
            logger.error("Adding new server configuration", e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
            return;
        }
        context.status(HttpStatus.CREATED);
    }

    @Override
    public void delete(@NotNull Context context, @NotNull String s) {
        try {
            this.userDBService.deleteConfig(Long.parseLong(s));
            context.status(HttpStatus.NO_CONTENT);
        } catch (DBException e) {
            logger.error("Deleting configuration with id: " + s, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void getAll(@NotNull Context context) {
        try {
            var allowedServers = this.userDBService.getAllConfigs();
            context.json(allowedServers);
        } catch (DBException e) {
            logger.error("Getting all user configs", e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void getOne(@NotNull Context context, @NotNull String s) {
        var id = Long.parseLong(s);
        try {
            var server = this.userDBService.getConfigBy(id);
            if (server.isEmpty()) {
                context.status(HttpStatus.NOT_FOUND);
                return;
            }
            context.json(server.get());
        } catch (DBException e) {
            logger.error("Searching for config with id: " + s, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public void update(@NotNull Context context, @NotNull String idString) {
        var body = context.bodyAsClass(UserConfig.NewUserConfig.class);
        var id = Long.parseLong(idString);

        try {
            this.userDBService.updateConfig(id, body);
        } catch (DBException e) {
            logger.error("Updating user config with id: " + idString, e);
            context.status(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
